function [ opticDiscMask ] = OpticDiscSegUsingCircularSections(retinalImage, xCenter, yCenter)
% Calculate optic disc segmentation of a retinal image

% opticDiscMask: generated optic disc mask

% retinalImage: input retinal image
% xCenter: x coordinate of the center of the retinal image
% yCenter: y coordinate of the center of the retinal image

nrows = size(retinalImage, 1);
nrows1 = nrows;
ncols = size(retinalImage, 2);
ncols1 = ncols;
retinalImageR = retinalImage(:, :, 1);
retinalImageG = retinalImage(:, :, 2);
retinalImageB = retinalImage(:, :, 3);

% The original retinal image is cropped around the specified center to provide a more convenient input image for segmentation of the optic disc
if nrows == 1536
    windowsize = 358;
elseif nrows == 1488
    windowsize = 358*0.3730/0.3930;
elseif nrows == 960
    windowsize = 358*0.3730/0.5960;
elseif nrows == 3744
    windowsize = 358*0.3730/0.1540;
elseif nrows == 570     % ONHSD images
    windowsize = 358*0.3730;
else
    error('Unexpected image dimensions')
end
ywindowmin = max(1, yCenter-round(windowsize/2)+1);
ywindowmax = min(nrows, yCenter+round(windowsize/2));
xwindowmin = max(1, xCenter-round(windowsize/2))+1;
xwindowmax = min(ncols, xCenter+round(windowsize/2));
retinalImageR = retinalImageR(ywindowmin: ywindowmax, xwindowmin: xwindowmax);
retinalImageG = retinalImageG(ywindowmin: ywindowmax, xwindowmin: xwindowmax);
retinalImageB = retinalImageB(ywindowmin: ywindowmax, xwindowmin:xwindowmax);
nrows = size(retinalImageR, 1);
ncols = size(retinalImageR, 2);

% An intensity image is obtained by summing up the three color channels
intensityImage = double(retinalImageR) + double(retinalImageG) + double(retinalImageB);
intensityImage = uint8(255*((intensityImage - min(intensityImage(:)))/(max(intensityImage(:)) - min(intensityImage(:)))));

% The intensity image is preprocessed to discard the unwanted regions at the limits of the retina if they exist
level = graythresh(intensityImage);
blackMask = intensityImage < level*255;
ccblackMask = bwconncomp(blackMask);
ccblackMasklabels = labelmatrix(ccblackMask);
ccblackMaskprops = regionprops(ccblackMasklabels, 'Area');
ccblackMaskAreas = [ ccblackMaskprops.Area ];
[ ~, indmaxccblackMaskAreas ] = max(ccblackMaskAreas);
blacMmask = ccblackMasklabels == indmaxccblackMaskAreas;
if mean(intensityImage(blacMmask)) < 10
    blackMaskleft = blacMmask;
    blackMaskleft(:, round(ncols/2):end) = 0;
    blackmaskright = blacMmask;
    blackmaskright(:, 1:round(ncols/2)) = 0;
    [ y, x ] = find(blackMaskleft);
    retinalImageRGBleftshift = circshift(intensityImage, [ 0 max(x) ]);
    [ y, x ] = find(blackmaskright);
    retinalImageRGBrightshift = circshift(intensityImage, [ 0 -(ncols-min(x)) ]);
    intensityImage(blackMaskleft) = retinalImageRGBleftshift(blackMaskleft);
    intensityImage(blackmaskright) = retinalImageRGBrightshift(blackmaskright);
end

% The optic disc mask is segmented
[ y, x ] = find(ones(size(intensityImage)));
x = reshape(x, nrows, ncols);
y = reshape(y, nrows, ncols);
x0 = round(ncols/2);
y0 = round(nrows/2);
rstart = 0.25*nrows;
rend = 0.45*nrows;
incr = 0.01*nrows;
offsetRadius = 0.04*nrows;
r = rstart:incr:rend;
meandifsleft = zeros(1, length(r));
meandifsright = zeros(1, length(r));
meandifsup = zeros(1, length(r));
meandifsdown = zeros(1, length(r));
for i = 1:length(r)
    circleMask = sqrt((y - y0).^2 + (x-x0).^2) <= r(i);
    outerMask = sqrt((y - y0).^2 + (x-x0).^2) <= r(i)+ offsetRadius;
    innerMask = sqrt((y - y0).^2 + (x-x0).^2) <= r(i) - offsetRadius;
    outerRegion = xor(circleMask, outerMask);
    [ yRegion, xRegion ] = find(outerRegion);
    outerregionleft = outerRegion(:, 1:mean(xRegion));
    outerRegionright = outerRegion(:, mean(xRegion)+1:ncols);
    outerRegionright = logical([ zeros(nrows, length(1:mean(xRegion))) outerRegionright ]);
    outerRegionup = outerRegion(1:mean(yRegion), :);
    outerRegionup = logical([ outerRegionup; zeros(length(mean(yRegion)+1:nrows), ncols);  ]);
    outerRegiondown = outerRegion(mean(yRegion)+1:nrows, :);
    outerRegiondown = logical([ zeros(length(1:mean(yRegion)), ncols); outerRegiondown ]);
    innerRegion = xor(circleMask, innerMask);
    [ yRegion, xRegion ] = find(innerRegion);
    innerRegionleft = innerRegion(:, 1:mean(xRegion));
    innerRegionright = innerRegion(:, mean(xRegion)+1:ncols);
    innerRegionright = logical([ zeros(nrows, length(1:mean(xRegion))) innerRegionright ]);
    innerRegionup = innerRegion(1:mean(yRegion), :);
    innerRegionup = logical([ innerRegionup; zeros(length(mean(yRegion)+1:nrows), ncols);  ]);
    innerRegiondown = innerRegion(mean(yRegion)+1:nrows, :);
    innerRegiondown = logical([ zeros(length(1:mean(yRegion)), ncols); innerRegiondown ]);
    meanOuterRegionleft = mean(intensityImage(outerregionleft));
    meanOuterRegionright = mean(intensityImage(outerRegionright));
    meanOuterRegionup = mean(intensityImage(outerRegionup));
    meanOuterRegiondown = mean(intensityImage(outerRegiondown));
    meanInnerRegionleft = mean(intensityImage(innerRegionleft));
    meanInnerRegionright = mean(intensityImage(innerRegionright));
    meanInnerRegionup = mean(intensityImage(innerRegionup));
    meanInnerRegiondown = mean(intensityImage(innerRegiondown));
    meandifsleft(i) = meanInnerRegionleft - meanOuterRegionleft;
    meandifsright(i) = meanInnerRegionright - meanOuterRegionright;
    meandifsup(i) = meanInnerRegionup - meanOuterRegionup;
    meandifsdown(i) = meanInnerRegiondown - meanOuterRegiondown;
end
[ ~, indmaxmeandifsleft ] = max(meandifsleft);
[ ~, indmaxmeandifsright ] = max(meandifsright);
[ ~, indmaxmeandifsup ] = max(meandifsup);
[ ~, indmaxmeandifsdown ] = max(meandifsdown);

a = (r(indmaxmeandifsleft) + r(indmaxmeandifsright))/2;
x0_1 = (x0 - r(indmaxmeandifsleft) + x0 + r(indmaxmeandifsright))/2;
b = (r(indmaxmeandifsup) + r(indmaxmeandifsdown))/2;
y0_1 = (y0 - r(indmaxmeandifsup) + y0 + r(indmaxmeandifsdown))/2;
[ y, x ] = find(ones(size(intensityImage)));
x = reshape(x, nrows, ncols);
y = reshape(y, nrows, ncols);
opticDiscMask = sqrt((y - y0_1).^2 + (x-x0_1).^2) <= mean([a b]);
opticDiscMask1 = zeros(nrows1, ncols1);
opticDiscMask1(ywindowmin:ywindowmax, xwindowmin:xwindowmax) = opticDiscMask;
opticDiscMask = opticDiscMask1;

end

